@@ -17,12 +17,18 @@
 package org.gradle.initialization;
 
 import org.gradle.api.internal.GradleInternal;
+import org.gradle.execution.EntryTaskSelector;
+import org.gradle.execution.plan.ExecutionPlan;
+import org.jspecify.annotations.Nullable;
+import org.gradle.internal.service.scopes.Scope;
+import org.gradle.internal.service.scopes.ServiceScope;
 
 /**
  * Responsible for preparing `Gradle` instances for task execution. The result is passed to a {@link org.gradle.execution.BuildWorkExecutor} for execution. Prior to preparing for task execution, the `Gradle` instance has its projects configured by a {@link org.gradle.configuration.ProjectsPreparer}.
  *
  * <p>This includes resolving the entry tasks and calculating the task graph.</p>
  */
+@ServiceScope(Scope.Build.class)
 public interface TaskExecutionPreparer {
-    void prepareForTaskExecution(GradleInternal gradle);
+    void scheduleRequestedTasks(GradleInternal gradle, @Nullable EntryTaskSelector selector, ExecutionPlan plan);
 }
