@@ -24,6 +24,7 @@ import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
 import org.gradle.api.artifacts.result.ResolvedVariantResult;
+import org.gradle.api.internal.artifacts.NamedVariantIdentifier;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.api.internal.attributes.immutable.ImmutableAttributesSchema;
 import org.gradle.internal.Describables;
@@ -48,11 +49,11 @@ import org.gradle.internal.component.model.ModuleSources;
 import org.gradle.internal.component.model.VariantArtifactResolveState;
 import org.gradle.internal.component.model.VariantGraphResolveMetadata;
 import org.gradle.internal.component.model.VariantGraphResolveState;
+import org.gradle.internal.component.model.VariantIdentifier;
 import org.gradle.internal.component.model.VariantResolveMetadata;
-import org.gradle.internal.deprecation.DeprecationLogger;
 import org.gradle.internal.lazy.Lazy;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -176,29 +177,8 @@ public class LenientPlatformGraphResolveState extends AbstractComponentGraphReso
             return Collections.emptyList();
         }
 
-        @Nullable
         @Override
         public VariantGraphResolveState getLegacyVariant() {
-            return doGetVariantByConfigurationName(Dependency.DEFAULT_CONFIGURATION);
-        }
-
-        @Nullable
-        @Override
-        public VariantGraphResolveState getVariantByConfigurationName(String name) {
-            DeprecationLogger.deprecateBehaviour("Selecting a variant by configuration name from a non-Ivy external component.")
-                .willBecomeAnErrorInGradle9()
-                .withUpgradeGuideSection(8, "selecting_variant_by_configuration_name")
-                .nagUser();
-
-            return doGetVariantByConfigurationName(name);
-        }
-
-        @Nullable
-        private VariantGraphResolveState doGetVariantByConfigurationName(String name) {
-            if (!name.equals(Dependency.DEFAULT_CONFIGURATION)) {
-                return null;
-            }
-
             return implicitVariantState.get();
         }
 
@@ -213,10 +193,12 @@ public class LenientPlatformGraphResolveState extends AbstractComponentGraphReso
                 component.getMetadata().getModuleVersionId()
             );
 
+            String name = Dependency.DEFAULT_CONFIGURATION;
+            NamedVariantIdentifier id = new NamedVariantIdentifier(component.getMetadata().getId(), name);
             return new LenientPlatformVariantGraphResolveState(
                 component.idGenerator.nextVariantId(),
                 component.getMetadata().getId(),
-                new LenientPlatformVariantGraphResolveMetadata(Dependency.DEFAULT_CONFIGURATION, false, dependencyFactory)
+                new LenientPlatformVariantGraphResolveMetadata(id, name, false, dependencyFactory)
             );
         }
 
@@ -236,10 +218,12 @@ public class LenientPlatformGraphResolveState extends AbstractComponentGraphReso
                 platformId
             );
 
+            String name = Dependency.DEFAULT_CONFIGURATION;
+            NamedVariantIdentifier id = new NamedVariantIdentifier(component.getMetadata().getId(), name);
             return new LenientPlatformVariantGraphResolveState(
                 component.idGenerator.nextVariantId(),
                 component.getMetadata().getId(),
-                new LenientPlatformVariantGraphResolveMetadata(Dependency.DEFAULT_CONFIGURATION, true, dependencyFactory)
+                new LenientPlatformVariantGraphResolveMetadata(id, name, true, dependencyFactory)
             );
         }
 
@@ -250,15 +234,18 @@ public class LenientPlatformGraphResolveState extends AbstractComponentGraphReso
      */
     private static class LenientPlatformVariantGraphResolveMetadata implements VariantGraphResolveMetadata {
 
+        private final VariantIdentifier id;
         private final String name;
         private final boolean transitive;
         private final VariantDependencyFactory dependencyFactory;
 
         public LenientPlatformVariantGraphResolveMetadata(
+            VariantIdentifier id,
             String name,
             boolean transitive,
             VariantDependencyFactory dependencyFactory
         ) {
+            this.id = id;
             this.name = name;
             this.transitive = transitive;
             this.dependencyFactory = dependencyFactory;
@@ -269,6 +256,11 @@ public class LenientPlatformGraphResolveState extends AbstractComponentGraphReso
             return name;
         }
 
+        @Override
+        public VariantIdentifier getId() {
+            return id;
+        }
+
         @Override
         public ImmutableAttributes getAttributes() {
             return ImmutableAttributes.EMPTY;
@@ -383,7 +375,7 @@ public class LenientPlatformGraphResolveState extends AbstractComponentGraphReso
             return ImmutableSet.of(new DefaultVariantMetadata(
                 name,
                 new ComponentConfigurationIdentifier(componentId, name),
-                Describables.of(componentId, "variant", name),
+                Describables.of(componentId, "variant", variant.getDisplayName()),
                 variant.getAttributes(),
                 ImmutableList.of(),
                 variant.getCapabilities()
