@@ -18,6 +18,7 @@ package org.gradle.kotlin.dsl.accessors
 import org.gradle.api.Action
 import org.gradle.api.Incubating
 import org.gradle.api.Project
+import org.gradle.api.internal.DynamicObjectAware
 import org.gradle.api.reflect.TypeOf
 import org.gradle.internal.deprecation.ConfigurationDeprecationType
 import org.gradle.internal.hash.Hashing.hashString
@@ -78,27 +79,33 @@ fun fragmentsFor(accessor: Accessor): Fragments = when (accessor) {
 }
 
 private fun fragmentsForSoftwareType(accessor: Accessor.ForSoftwareType): Fragments = accessor.run {
-    val className = "${accessor.spec.softwareTypeName.original.uppercaseFirstChar()}ContainerElementFactoriesKt"
-    val functionName = spec.softwareTypeName.original
-    val (kotlinProjectType, jvmProjectType) = accessibleTypesFor(TypeAccessibility.Accessible(SchemaType.of<Project>(), emptyList()))
+    val className = "${accessor.spec.softwareFeatureName.original.uppercaseFirstChar()}ContainerElementFactoriesKt"
+    val functionName = spec.softwareFeatureName.original
     val (kotlinModelType, _) = accessibleTypesFor(accessor.spec.modelType)
-    val deprecation = accessor.spec.modelType.deprecation()
-    val annotations = "${maybeDeprecationAnnotations(deprecation)}${maybeOptInAnnotationSource(accessor.spec.modelType)}"
+    val (kotlinTargetType, jvmTargetType) = accessibleTypesFor(accessor.spec.targetType)
+    val deprecation = highestDeprecationByLevel(accessor.spec.modelType.deprecation(), accessor.spec.targetType.deprecation())
+    val annotations = "${maybeDeprecationAnnotations(deprecation)}${maybeOptInAnnotationSource(accessor.spec.modelType, accessor.spec.targetType)}"
+
+    val targetTypeKotlinString = spec.targetType.type.kotlinString
+    val featureKind = when (accessor.spec.targetType.type.value.concreteClass) {
+        Project::class.java -> "project type"
+        else -> "project feature"
+    }
 
     className to sequenceOf(
         AccessorFragment(
             source = """
             |        /**
-            |         * Applies the "$functionName" software type to the project and configures the model with the [configure] action.
+            |         * Applies the "$functionName" $featureKind to the target and configures the definition with the [configure] action.
             |         */
             |        @Incubating
-            |        ${annotations}fun Project.`${functionName}`(configure: Action<in ${spec.modelType.type.kotlinString}>) {
+            |        ${annotations}fun $targetTypeKotlinString.`${functionName}`(configure: Action<in ${spec.modelType.type.kotlinString}>) {
             |            applySoftwareType(this, "$functionName", configure)
             |        }
             """.trimMargin(),
             signature = JvmMethodSignature(
                 functionName,
-                "(L$jvmProjectType;Lorg/gradle/api/Action;)V"
+                "(L$jvmTargetType;Lorg/gradle/api/Action;)V"
             ),
             bytecode = {
                 publicStaticMethod(signature, annotations = {
@@ -106,16 +113,17 @@ private fun fragmentsForSoftwareType(accessor: Accessor.ForSoftwareType): Fragme
                 }) {
                     maybeWithDeprecation(deprecation)
                     ALOAD(0)
+                    CHECKCAST(DynamicObjectAware::class.internalName)
                     LDC(functionName)
                     ALOAD(1)
-                    invokeRuntime("applySoftwareType", "(L${Project::class.internalName};L${String::class.internalName};L${Action::class.internalName};)V")
+                    invokeRuntime("applySoftwareFeature", "(L${DynamicObjectAware::class.internalName};L${String::class.internalName};L${Action::class.internalName};)V")
                     RETURN()
                 }
             },
             metadata = {
                 kmPackage.functions += newFunctionOf(
                     functionAttributes = publicFunctionWithAnnotationsAttributes, // has @Incubating and maybe deprecations
-                    receiverType = kotlinProjectType,
+                    receiverType = kotlinTargetType,
                     valueParameters = listOf(
                         newValueParameterOf("configure", newClassTypeOf(Action::class.java.name.replace(".", "/"), KmTypeProjection(KmVariance.IN, kotlinModelType)))
                     ),
@@ -1051,7 +1059,7 @@ fun fragmentsForModelDefault(
     val accessorSpec = accessor.spec
     val className = internalNameForAccessorClassOf(accessorSpec)
     val (accessibleReceiverType, name, modelType) = accessorSpec
-    val softwareTypeName = name.kotlinIdentifier
+    val softwareFeatureName = name.kotlinIdentifier
     val receiverType = accessibleReceiverType.type.kmType
     val (kotlinPublicType, jvmPublicType) = accessibleTypesFor(modelType)
     val deprecation = accessor.spec.type.deprecation()
@@ -1065,7 +1073,7 @@ fun fragmentsForModelDefault(
                     maybeWithDeprecation(deprecation)
                     maybeWithOptInRequirement(optIns)
                     ALOAD(0)
-                    LDC(softwareTypeName)
+                    LDC(softwareFeatureName)
                     LDC(jvmPublicType)
                     ALOAD(1)
                     INVOKEINTERFACE(GradleTypeName.modeDefaults, "add", "(Ljava/lang/String;Ljava/lang/Class;Lorg/gradle/api/Action;)V")
@@ -1076,7 +1084,7 @@ fun fragmentsForModelDefault(
                 kmPackage.functions += newFunctionOf(
                     receiverType = receiverType,
                     returnType = KotlinType.unit,
-                    name = softwareTypeName,
+                    name = softwareFeatureName,
                     valueParameters = listOf(
                         newValueParameterOf("configureAction", actionTypeOf(kotlinPublicType))
                     ),
@@ -1128,6 +1136,9 @@ internal fun TypeAccessibility.deprecation(): Deprecated? =
         else -> null
     }
 
+internal fun highestDeprecationByLevel(deprecated: Deprecated?, other: Deprecated?): Deprecated? =
+    listOfNotNull(deprecated, other).maxByOrNull { it.level }
+
 internal fun TypeAccessibility.requiredOptIns(): List<AnnotationRepresentation>? =
     when (this) {
         is TypeAccessibility.Accessible -> this.optInRequirements.takeIf { it.isNotEmpty() }
@@ -1158,13 +1169,24 @@ val TypeOf<*>.kmType: KmType
     get() = when {
         isParameterized -> genericTypeOf(
             classOf(parameterizedTypeDefinition.concreteClass),
-            actualTypeArguments.map { it.kmType }
+            actualTypeArguments.map { it.kmTypeProjection }
         )
 
         isWildcard -> (upperBound ?: lowerBound)?.kmType ?: KotlinType.any
         else -> classOf(concreteClass)
     }
 
+private
+val TypeOf<*>.kmTypeProjection: KmTypeProjection
+    get() = KmTypeProjection(
+        variance = when {
+            upperBound != null -> KmVariance.OUT
+            lowerBound != null -> KmVariance.IN
+            else -> KmVariance.INVARIANT
+        },
+        type = kmType
+    )
+
 
 internal
 inline fun <reified T> classOf(): KmType =
