@@ -21,18 +21,20 @@ import org.gradle.api.Action;
 import org.gradle.api.JavaVersion;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.ConventionTask;
+import org.gradle.api.internal.provider.PropertyFactory;
 import org.gradle.api.jvm.ModularitySpec;
 import org.gradle.api.model.ObjectFactory;
-import org.gradle.api.model.ReplacedBy;
+import org.gradle.api.provider.ListProperty;
 import org.gradle.api.provider.Property;
 import org.gradle.api.provider.Provider;
 import org.gradle.api.provider.ProviderFactory;
+import org.gradle.api.tasks.internal.JavaExecExecutableUtils;
 import org.gradle.api.tasks.options.Option;
-import org.gradle.internal.deprecation.DeprecationLogger;
+import org.gradle.internal.instrumentation.api.annotations.ToBeReplacedByLazyProperty;
 import org.gradle.internal.jvm.DefaultModularitySpec;
-import org.gradle.internal.jvm.Jvm;
-import org.gradle.internal.jvm.inspection.JvmVersionDetector;
 import org.gradle.jvm.toolchain.JavaLauncher;
+import org.gradle.jvm.toolchain.JavaToolchainService;
+import org.gradle.jvm.toolchain.internal.JavaExecutableUtils;
 import org.gradle.process.CommandLineArgumentProvider;
 import org.gradle.process.ExecResult;
 import org.gradle.process.JavaDebugOptions;
@@ -43,8 +45,8 @@ import org.gradle.process.internal.DefaultJavaExecSpec;
 import org.gradle.process.internal.ExecActionFactory;
 import org.gradle.process.internal.JavaExecAction;
 import org.gradle.work.DisableCachingByDefault;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
 import javax.inject.Inject;
 import java.io.File;
 import java.io.InputStream;
@@ -53,6 +55,8 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
 
+import static java.util.Collections.emptyList;
+
 /**
  * Executes a Java application in a child process.
  * <p>
@@ -111,7 +115,8 @@ import java.util.Map;
  * </pre>
  */
 @DisableCachingByDefault(because = "Gradle would require more information to cache this task")
-public class JavaExec extends ConventionTask implements JavaExecSpec {
+public abstract class JavaExec extends ConventionTask implements JavaExecSpec {
+
     private final DefaultJavaExecSpec javaExecSpec;
     private final Property<String> mainModule;
     private final Property<String> mainClass;
@@ -125,43 +130,55 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
         mainClass = objectFactory.property(String.class);
         modularity = objectFactory.newInstance(DefaultModularitySpec.class);
         execResult = objectFactory.property(ExecResult.class);
-
         javaExecSpec = objectFactory.newInstance(DefaultJavaExecSpec.class);
-        javaExecSpec.getMainClass().convention(getMainClass().orElse(getProviderFactory()
-            .provider(() -> DeprecationLogger.whileDisabled(this::getMain)))); // go through 'main' to keep this compatible with existing convention mappings
+
+        Provider<Iterable<String>> jvmArgumentsConvention = getProviderFactory().provider(this::jvmArgsConventionValue);
+        javaExecSpec.getJvmArguments().convention(jvmArgumentsConvention);
+
+        javaExecSpec.getMainClass().convention(mainClass);
         javaExecSpec.getMainModule().convention(mainModule);
         javaExecSpec.getModularity().getInferModulePath().convention(modularity.getInferModulePath());
-        javaLauncher = objectFactory.property(JavaLauncher.class);
-    }
 
-    @Inject
-    protected ObjectFactory getObjectFactory() {
-        throw new UnsupportedOperationException();
-    }
-
-    @Inject
-    protected ExecActionFactory getExecActionFactory() {
-        throw new UnsupportedOperationException();
-    }
-
-    @Inject
-    protected ProviderFactory getProviderFactory() {
-        throw new UnsupportedOperationException();
+        JavaToolchainService javaToolchainService = getJavaToolchainService();
+        PropertyFactory propertyFactory = getPropertyFactory();
+        Provider<JavaLauncher> javaLauncherConvention = getProviderFactory()
+            .provider(() -> JavaExecExecutableUtils.getExecutableOverrideToolchainSpec(this, propertyFactory))
+            .flatMap(javaToolchainService::launcherFor)
+            .orElse(javaToolchainService.launcherFor(it -> {}));
+        javaLauncher = objectFactory.property(JavaLauncher.class).convention(javaLauncherConvention);
+        javaLauncher.finalizeValueOnRead();
     }
 
     @TaskAction
     public void exec() {
-        setJvmArgs(getJvmArgs()); // convention mapping for 'jvmArgs'
+        validateExecutableMatchesToolchain();
+
+        List<String> jvmArgs = javaExecSpec.getJvmArguments().getOrNull();
+        if (jvmArgs != null) {
+            javaExecSpec.setExtraJvmArgs(jvmArgs);
+        }
+
         JavaExecAction javaExecAction = getExecActionFactory().newJavaExecAction();
         javaExecSpec.copyTo(javaExecAction);
-        javaExecAction.setExecutable(getEffectiveExecutable());
+        String effectiveExecutable = getJavaLauncher().get().getExecutablePath().toString();
+        javaExecAction.setExecutable(effectiveExecutable);
+
         execResult.set(javaExecAction.execute());
     }
 
+    private void validateExecutableMatchesToolchain() {
+        File toolchainExecutable = getJavaLauncher().get().getExecutablePath().getAsFile();
+        String customExecutable = getExecutable();
+        JavaExecutableUtils.validateExecutable(
+            customExecutable, "Toolchain from `executable` property",
+            toolchainExecutable, "toolchain from `javaLauncher` property");
+    }
+
     /**
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public List<String> getAllJvmArgs() {
         return javaExecSpec.getAllJvmArgs();
     }
@@ -186,8 +203,9 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public List<String> getJvmArgs() {
-        return javaExecSpec.getJvmArgs();
+        return javaExecSpec.getJvmArguments().get();
     }
 
     /**
@@ -195,7 +213,8 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     public void setJvmArgs(List<String> arguments) {
-        javaExecSpec.setJvmArgs(arguments);
+        javaExecSpec.getJvmArguments().empty();
+        jvmArgs(arguments);
     }
 
     /**
@@ -203,7 +222,8 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     public void setJvmArgs(Iterable<?> arguments) {
-        javaExecSpec.setJvmArgs(arguments);
+        javaExecSpec.getJvmArguments().empty();
+        jvmArgs(arguments);
     }
 
     /**
@@ -211,16 +231,23 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     public JavaExec jvmArgs(Iterable<?> arguments) {
-        javaExecSpec.jvmArgs(arguments);
+        addJvmArguments(arguments);
+        javaExecSpec.checkDebugConfiguration(arguments);
         return this;
     }
 
+    private void addJvmArguments(Iterable<?> arguments) {
+        for (Object arg : arguments) {
+            javaExecSpec.getJvmArguments().add(arg.toString());
+        }
+    }
+
     /**
      * {@inheritDoc}
      */
     @Override
     public JavaExec jvmArgs(Object... arguments) {
-        javaExecSpec.jvmArgs(arguments);
+        jvmArgs(Arrays.asList(arguments));
         return this;
     }
 
@@ -228,7 +255,8 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public Map<String, Object> getSystemProperties() {
+    @ToBeReplacedByLazyProperty
+    public Map<String, @Nullable Object> getSystemProperties() {
         return javaExecSpec.getSystemProperties();
     }
 
@@ -236,7 +264,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public void setSystemProperties(Map<String, ?> properties) {
+    public void setSystemProperties(Map<String, ? extends @Nullable Object> properties) {
         javaExecSpec.setSystemProperties(properties);
     }
 
@@ -244,7 +272,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public JavaExec systemProperties(Map<String, ?> properties) {
+    public JavaExec systemProperties(Map<String, ? extends @Nullable Object> properties) {
         javaExecSpec.systemProperties(properties);
         return this;
     }
@@ -253,7 +281,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public JavaExec systemProperty(String name, Object value) {
+    public JavaExec systemProperty(String name, @Nullable Object value) {
         javaExecSpec.systemProperty(name, value);
         return this;
     }
@@ -262,6 +290,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public FileCollection getBootstrapClasspath() {
         return javaExecSpec.getBootstrapClasspath();
     }
@@ -278,7 +307,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public JavaExec bootstrapClasspath(Object... classpath) {
+    public JavaExec bootstrapClasspath(@Nullable Object... classpath) {
         javaExecSpec.bootstrapClasspath(classpath);
         return this;
     }
@@ -287,6 +316,8 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
+    @Nullable
     public String getMinHeapSize() {
         return javaExecSpec.getMinHeapSize();
     }
@@ -295,7 +326,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public void setMinHeapSize(String heapSize) {
+    public void setMinHeapSize(@Nullable String heapSize) {
         javaExecSpec.setMinHeapSize(heapSize);
     }
 
@@ -303,6 +334,8 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
+    @Nullable
     public String getDefaultCharacterEncoding() {
         return javaExecSpec.getDefaultCharacterEncoding();
     }
@@ -311,7 +344,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public void setDefaultCharacterEncoding(String defaultCharacterEncoding) {
+    public void setDefaultCharacterEncoding(@Nullable String defaultCharacterEncoding) {
         javaExecSpec.setDefaultCharacterEncoding(defaultCharacterEncoding);
     }
 
@@ -319,6 +352,8 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
+    @Nullable
     public String getMaxHeapSize() {
         return javaExecSpec.getMaxHeapSize();
     }
@@ -327,7 +362,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public void setMaxHeapSize(String heapSize) {
+    public void setMaxHeapSize(@Nullable String heapSize) {
         javaExecSpec.setMaxHeapSize(heapSize);
     }
 
@@ -335,6 +370,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public boolean getEnableAssertions() {
         return javaExecSpec.getEnableAssertions();
     }
@@ -351,6 +387,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public boolean getDebug() {
         return javaExecSpec.getDebug();
     }
@@ -364,11 +401,17 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
         javaExecSpec.setDebug(enabled);
     }
 
+    /**
+     * {@inheritDoc}
+     */
     @Override
     public JavaDebugOptions getDebugOptions() {
         return javaExecSpec.getDebugOptions();
     }
 
+    /**
+     * {@inheritDoc}
+     */
     @Override
     public void debugOptions(Action<JavaDebugOptions> action) {
         javaExecSpec.debugOptions(action);
@@ -395,38 +438,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    @Deprecated
-    @ReplacedBy("mainClass")
-    public String getMain() {
-        DeprecationLogger.deprecateProperty(JavaExec.class, "main")
-            .replaceWith("mainClass")
-            .willBeRemovedInGradle8()
-            .withDslReference()
-            .nagUser();
-
-        return getMainClass().getOrNull();
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    @Deprecated
-    public JavaExec setMain(String mainClassName) {
-        DeprecationLogger.deprecateProperty(JavaExec.class, "main")
-            .replaceWith("mainClass")
-            .willBeRemovedInGradle8()
-            .withDslReference()
-            .nagUser();
-
-        getMainClass().set(mainClassName);
-        return this;
-    }
-
-    /**
-     * {@inheritDoc}
-     */
-    @Override
+    @ToBeReplacedByLazyProperty
     public List<String> getArgs() {
         return javaExecSpec.getArgs();
     }
@@ -493,6 +505,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public List<CommandLineArgumentProvider> getArgumentProviders() {
         return javaExecSpec.getArgumentProviders();
     }
@@ -510,7 +523,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
-    public JavaExec classpath(Object... paths) {
+    public JavaExec classpath(@Nullable Object... paths) {
         javaExecSpec.classpath(paths);
         return this;
     }
@@ -519,6 +532,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public FileCollection getClasspath() {
         return javaExecSpec.getClasspath();
     }
@@ -541,13 +555,14 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
     }
 
     /**
-     * Returns the version of the Java executable specified by {@link #getExecutable()}.
+     * Returns the version of the Java executable specified by {@link #getJavaLauncher()}.
      *
      * @since 5.2
      */
     @Input
+    @ToBeReplacedByLazyProperty
     public JavaVersion getJavaVersion() {
-        return getServices().get(JvmVersionDetector.class).getJavaVersion(getEffectiveExecutable());
+        return JavaVersion.toVersion(getJavaLauncher().get().getMetadata().getLanguageVersion().asInt());
     }
 
     /**
@@ -556,6 +571,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
     @Internal("covered by getJavaVersion")
     @Nullable
     @Override
+    @ToBeReplacedByLazyProperty
     public String getExecutable() {
         return javaExecSpec.getExecutable();
     }
@@ -590,6 +606,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Internal
+    @ToBeReplacedByLazyProperty
     public File getWorkingDir() {
         return javaExecSpec.getWorkingDir();
     }
@@ -624,6 +641,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Internal
+    @ToBeReplacedByLazyProperty
     public Map<String, Object> getEnvironment() {
         return javaExecSpec.getEnvironment();
     }
@@ -677,6 +695,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Internal
+    @ToBeReplacedByLazyProperty
     public InputStream getStandardInput() {
         return javaExecSpec.getStandardInput();
     }
@@ -695,6 +714,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Internal
+    @ToBeReplacedByLazyProperty
     public OutputStream getStandardOutput() {
         return javaExecSpec.getStandardOutput();
     }
@@ -713,6 +733,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Internal
+    @ToBeReplacedByLazyProperty
     public OutputStream getErrorOutput() {
         return javaExecSpec.getErrorOutput();
     }
@@ -731,6 +752,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Input
+    @ToBeReplacedByLazyProperty
     public boolean isIgnoreExitValue() {
         return javaExecSpec.isIgnoreExitValue();
     }
@@ -740,6 +762,7 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      */
     @Override
     @Internal
+    @ToBeReplacedByLazyProperty
     public List<String> getCommandLine() {
         return javaExecSpec.getCommandLine();
     }
@@ -748,10 +771,19 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * {@inheritDoc}
      */
     @Override
+    @ToBeReplacedByLazyProperty
     public List<CommandLineArgumentProvider> getJvmArgumentProviders() {
         return javaExecSpec.getJvmArgumentProviders();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public ListProperty<String> getJvmArguments() {
+        return javaExecSpec.getJvmArguments();
+    }
+
     /**
      * Returns the result for the command run by this task. The provider has no value if this task has not been executed yet.
      *
@@ -769,20 +801,27 @@ public class JavaExec extends ConventionTask implements JavaExecSpec {
      * @since 6.7
      */
     @Nested
-    @Optional
     public Property<JavaLauncher> getJavaLauncher() {
         return javaLauncher;
     }
 
-    private String getEffectiveExecutable() {
-        if (javaLauncher.isPresent()) {
-            return javaLauncher.get().getExecutablePath().toString();
-        }
-        final String executable = getExecutable();
-        if (executable != null) {
-            return executable;
-        }
-        return Jvm.current().getJavaExecutable().getAbsolutePath();
-    }
+    @Inject
+    protected abstract ObjectFactory getObjectFactory();
 
+    @Inject
+    protected abstract PropertyFactory getPropertyFactory();
+
+    @Inject
+    protected abstract ExecActionFactory getExecActionFactory();
+
+    @Inject
+    protected abstract JavaToolchainService getJavaToolchainService();
+
+    @Inject
+    protected abstract ProviderFactory getProviderFactory();
+
+    private Iterable<String> jvmArgsConventionValue() {
+        Iterable<String> jvmArgs = getConventionMapping().getConventionValue(null, "jvmArgs", false);
+        return jvmArgs != null ? jvmArgs : emptyList();
+    }
 }
