@@ -26,7 +26,6 @@ import com.google.common.collect.Ordering;
 import com.google.common.util.concurrent.UncheckedExecutionException;
 import org.gradle.internal.Cast;
 import org.gradle.internal.Factory;
-import org.gradle.internal.InternalTransformer;
 import org.gradle.internal.UncheckedException;
 import org.gradle.internal.reflect.GroovyMethods;
 import org.gradle.internal.service.scopes.Scope;
@@ -98,12 +97,7 @@ public class ModelRuleExtractor {
     }
 
     private String describeHandlers() {
-        String desc = Joiner.on(", ").join(CollectionUtils.collect(handlers, new InternalTransformer<String, MethodModelRuleExtractor>() {
-            @Override
-            public String transform(MethodModelRuleExtractor original) {
-                return original.getDescription();
-            }
-        }));
+        String desc = Joiner.on(", ").join(CollectionUtils.collect(handlers, MethodModelRuleExtractor::getDescription));
         return "[" + desc + "]";
     }
 
@@ -400,12 +394,7 @@ public class ModelRuleExtractor {
 
         @VisibleForTesting // used in tests only
         public List<ExtractedModelRule> getRules() {
-            return CollectionUtils.collect(rules, new InternalTransformer<ExtractedModelRule, ExtractedRuleDetails>() {
-                @Override
-                public ExtractedModelRule transform(ExtractedRuleDetails extractedRuleDetails) {
-                    return extractedRuleDetails.rule;
-                }
-            });
+            return CollectionUtils.collect(rules, extractedRuleDetails -> extractedRuleDetails.rule);
         }
 
         @Override
