@@ -17,7 +17,7 @@
 package org.gradle.jvm.application.scripts;
 
 import org.gradle.api.resources.TextResource;
-import org.gradle.internal.instrumentation.api.annotations.ToBeReplacedByLazyProperty;
+import org.gradle.api.tasks.Nested;
 
 /**
  * Interface for generating scripts with the provided details based on a provided template.
@@ -36,7 +36,7 @@ public interface TemplateBasedScriptGenerator extends ScriptGenerator {
      *
      * @return Template reader
      */
-    @ToBeReplacedByLazyProperty
+    @Nested
     TextResource getTemplate();
 
 }
