@@ -17,5 +17,5 @@
 /**
  * Reporting components used by software reports.
  */
-@org.gradle.api.NonNullApi
+@org.jspecify.annotations.NullMarked
 package org.gradle.api.reporting.components;
