@@ -66,28 +66,36 @@ import org.gradle.tooling.events.lifecycle.internal.DefaultBuildPhaseFinishEvent
 import org.gradle.tooling.events.lifecycle.internal.DefaultBuildPhaseOperationDescriptor;
 import org.gradle.tooling.events.lifecycle.internal.DefaultBuildPhaseStartEvent;
 import org.gradle.tooling.events.problems.AdditionalData;
+import org.gradle.tooling.events.problems.ContextualLabel;
 import org.gradle.tooling.events.problems.Details;
 import org.gradle.tooling.events.problems.DocumentationLink;
-import org.gradle.tooling.events.problems.FailureContainer;
-import org.gradle.tooling.events.problems.Label;
 import org.gradle.tooling.events.problems.Location;
-import org.gradle.tooling.events.problems.ProblemCategory;
+import org.gradle.tooling.events.problems.Problem;
 import org.gradle.tooling.events.problems.ProblemContext;
+import org.gradle.tooling.events.problems.ProblemDefinition;
 import org.gradle.tooling.events.problems.ProblemEvent;
+import org.gradle.tooling.events.problems.ProblemGroup;
+import org.gradle.tooling.events.problems.ProblemId;
+import org.gradle.tooling.events.problems.ProblemSummary;
 import org.gradle.tooling.events.problems.Severity;
 import org.gradle.tooling.events.problems.Solution;
 import org.gradle.tooling.events.problems.internal.DefaultAdditionalData;
-import org.gradle.tooling.events.problems.internal.DefaultCategory;
+import org.gradle.tooling.events.problems.internal.DefaultContextualLabel;
+import org.gradle.tooling.events.problems.internal.DefaultCustomAdditionalData;
 import org.gradle.tooling.events.problems.internal.DefaultDetails;
 import org.gradle.tooling.events.problems.internal.DefaultDocumentationLink;
-import org.gradle.tooling.events.problems.internal.DefaultFailureContainer;
 import org.gradle.tooling.events.problems.internal.DefaultFileLocation;
-import org.gradle.tooling.events.problems.internal.DefaultLabel;
 import org.gradle.tooling.events.problems.internal.DefaultLineInFileLocation;
 import org.gradle.tooling.events.problems.internal.DefaultOffsetInFileLocation;
 import org.gradle.tooling.events.problems.internal.DefaultPluginIdLocation;
+import org.gradle.tooling.events.problems.internal.DefaultProblem;
 import org.gradle.tooling.events.problems.internal.DefaultProblemAggregation;
 import org.gradle.tooling.events.problems.internal.DefaultProblemAggregationEvent;
+import org.gradle.tooling.events.problems.internal.DefaultProblemDefinition;
+import org.gradle.tooling.events.problems.internal.DefaultProblemGroup;
+import org.gradle.tooling.events.problems.internal.DefaultProblemId;
+import org.gradle.tooling.events.problems.internal.DefaultProblemSummariesEvent;
+import org.gradle.tooling.events.problems.internal.DefaultProblemSummary;
 import org.gradle.tooling.events.problems.internal.DefaultProblemsOperationContext;
 import org.gradle.tooling.events.problems.internal.DefaultSeverity;
 import org.gradle.tooling.events.problems.internal.DefaultSingleProblemEvent;
@@ -111,6 +119,7 @@ import org.gradle.tooling.events.task.java.JavaCompileTaskOperationResult.Annota
 import org.gradle.tooling.events.test.Destination;
 import org.gradle.tooling.events.test.JvmTestKind;
 import org.gradle.tooling.events.test.TestFinishEvent;
+import org.gradle.tooling.events.test.TestMetadataEvent;
 import org.gradle.tooling.events.test.TestOperationDescriptor;
 import org.gradle.tooling.events.test.TestOperationResult;
 import org.gradle.tooling.events.test.TestOutputDescriptor;
@@ -120,6 +129,7 @@ import org.gradle.tooling.events.test.TestStartEvent;
 import org.gradle.tooling.events.test.internal.DefaultJvmTestOperationDescriptor;
 import org.gradle.tooling.events.test.internal.DefaultTestFailureResult;
 import org.gradle.tooling.events.test.internal.DefaultTestFinishEvent;
+import org.gradle.tooling.events.test.internal.DefaultTestMetadataEvent;
 import org.gradle.tooling.events.test.internal.DefaultTestOperationDescriptor;
 import org.gradle.tooling.events.test.internal.DefaultTestOutputEvent;
 import org.gradle.tooling.events.test.internal.DefaultTestOutputOperationDescriptor;
@@ -150,13 +160,23 @@ import org.gradle.tooling.internal.consumer.DefaultFailure;
 import org.gradle.tooling.internal.consumer.DefaultFileComparisonTestAssertionFailure;
 import org.gradle.tooling.internal.consumer.DefaultTestAssertionFailure;
 import org.gradle.tooling.internal.consumer.DefaultTestFrameworkFailure;
+import org.gradle.tooling.internal.protocol.InternalBasicProblemDetailsVersion3;
+import org.gradle.tooling.internal.protocol.InternalBasicProblemDetailsVersion4;
 import org.gradle.tooling.internal.protocol.InternalBuildProgressListener;
 import org.gradle.tooling.internal.protocol.InternalFailure;
 import org.gradle.tooling.internal.protocol.InternalFileComparisonTestAssertionFailure;
 import org.gradle.tooling.internal.protocol.InternalProblemAggregationDetailsV2;
+import org.gradle.tooling.internal.protocol.InternalProblemAggregationDetailsVersion3;
 import org.gradle.tooling.internal.protocol.InternalProblemContextDetails;
+import org.gradle.tooling.internal.protocol.InternalProblemContextDetailsV2;
+import org.gradle.tooling.internal.protocol.InternalProblemDefinition;
 import org.gradle.tooling.internal.protocol.InternalProblemDetails;
 import org.gradle.tooling.internal.protocol.InternalProblemEvent;
+import org.gradle.tooling.internal.protocol.InternalProblemEventVersion2;
+import org.gradle.tooling.internal.protocol.InternalProblemGroup;
+import org.gradle.tooling.internal.protocol.InternalProblemId;
+import org.gradle.tooling.internal.protocol.InternalProblemSummariesDetails;
+import org.gradle.tooling.internal.protocol.InternalProblemSummary;
 import org.gradle.tooling.internal.protocol.InternalTestAssertionFailure;
 import org.gradle.tooling.internal.protocol.InternalTestFrameworkFailure;
 import org.gradle.tooling.internal.protocol.events.InternalBinaryPluginIdentifier;
@@ -179,6 +199,7 @@ import org.gradle.tooling.internal.protocol.events.InternalProgressEvent;
 import org.gradle.tooling.internal.protocol.events.InternalProjectConfigurationDescriptor;
 import org.gradle.tooling.internal.protocol.events.InternalProjectConfigurationResult;
 import org.gradle.tooling.internal.protocol.events.InternalProjectConfigurationResult.InternalPluginApplicationResult;
+import org.gradle.tooling.internal.protocol.events.InternalRootOperationDescriptor;
 import org.gradle.tooling.internal.protocol.events.InternalScriptPluginIdentifier;
 import org.gradle.tooling.internal.protocol.events.InternalStatusEvent;
 import org.gradle.tooling.internal.protocol.events.InternalSuccessResult;
@@ -192,6 +213,8 @@ import org.gradle.tooling.internal.protocol.events.InternalTaskWithExtraInfoDesc
 import org.gradle.tooling.internal.protocol.events.InternalTestDescriptor;
 import org.gradle.tooling.internal.protocol.events.InternalTestFailureResult;
 import org.gradle.tooling.internal.protocol.events.InternalTestFinishedProgressEvent;
+import org.gradle.tooling.internal.protocol.events.InternalTestMetadataDescriptor;
+import org.gradle.tooling.internal.protocol.events.InternalTestMetadataEvent;
 import org.gradle.tooling.internal.protocol.events.InternalTestOutputDescriptor;
 import org.gradle.tooling.internal.protocol.events.InternalTestOutputEvent;
 import org.gradle.tooling.internal.protocol.events.InternalTestProgressEvent;
@@ -204,6 +227,7 @@ import org.gradle.tooling.internal.protocol.events.InternalWorkItemDescriptor;
 import org.gradle.tooling.internal.protocol.problem.InternalAdditionalData;
 import org.gradle.tooling.internal.protocol.problem.InternalBasicProblemDetails;
 import org.gradle.tooling.internal.protocol.problem.InternalBasicProblemDetailsVersion2;
+import org.gradle.tooling.internal.protocol.problem.InternalContextualLabel;
 import org.gradle.tooling.internal.protocol.problem.InternalDetails;
 import org.gradle.tooling.internal.protocol.problem.InternalDocumentationLink;
 import org.gradle.tooling.internal.protocol.problem.InternalFileLocation;
@@ -213,13 +237,16 @@ import org.gradle.tooling.internal.protocol.problem.InternalLocation;
 import org.gradle.tooling.internal.protocol.problem.InternalOffsetInFileLocation;
 import org.gradle.tooling.internal.protocol.problem.InternalPluginIdLocation;
 import org.gradle.tooling.internal.protocol.problem.InternalProblemCategory;
+import org.gradle.tooling.internal.protocol.problem.InternalProblemDetailsVersion2;
+import org.gradle.tooling.internal.protocol.problem.InternalProxiedAdditionalData;
 import org.gradle.tooling.internal.protocol.problem.InternalSeverity;
 import org.gradle.tooling.internal.protocol.problem.InternalSolution;
 import org.gradle.tooling.internal.protocol.problem.InternalTaskPathLocation;
+import org.jspecify.annotations.NonNull;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nonnull;
-import javax.annotation.Nullable;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
@@ -232,6 +259,8 @@ import static java.util.Collections.emptyList;
 /**
  * Converts progress events sent from the tooling provider to the tooling client to the corresponding event types available on the public Tooling API, and broadcasts the converted events to the
  * matching progress listeners. This adapter handles all the different incoming progress event types (except the original logging-derived progress listener).
+ *
+ * This adapts tooling provider internal types into the public types on the consumer.
  */
 public class BuildProgressListenerAdapter implements InternalBuildProgressListener {
 
@@ -242,14 +271,15 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
     private final ListenerBroadcast<ProgressListener> projectConfigurationProgressListeners = new ListenerBroadcast<>(ProgressListener.class);
     private final ListenerBroadcast<ProgressListener> transformProgressListeners = new ListenerBroadcast<>(ProgressListener.class);
     private final ListenerBroadcast<ProgressListener> testOutputProgressListeners = new ListenerBroadcast<>(ProgressListener.class);
+    private final ListenerBroadcast<ProgressListener> testMetadataProgressListeners = new ListenerBroadcast<>(ProgressListener.class);
     private final ListenerBroadcast<ProgressListener> fileDownloadListeners = new ListenerBroadcast<>(ProgressListener.class);
     private final ListenerBroadcast<ProgressListener> buildPhaseListeners = new ListenerBroadcast<>(ProgressListener.class);
     private final ListenerBroadcast<ProgressListener> problemListeners = new ListenerBroadcast<>(ProgressListener.class);
+    private final ListenerBroadcast<ProgressListener> rootBuildListeners = new ListenerBroadcast<>(ProgressListener.class);
 
     private final Map<Object, OperationDescriptor> descriptorCache = new HashMap<>();
 
-    BuildProgressListenerAdapter(Map<OperationType, List<ProgressListener>> listeners) {
-
+    public BuildProgressListenerAdapter(Map<OperationType, List<ProgressListener>> listeners) {
         testProgressListeners.addAll(getOrDefault(listeners, OperationType.TEST));
         taskProgressListeners.addAll(getOrDefault(listeners, OperationType.TASK));
         buildOperationProgressListeners.addAll(getOrDefault(listeners, OperationType.GENERIC));
@@ -257,9 +287,11 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         projectConfigurationProgressListeners.addAll(getOrDefault(listeners, OperationType.PROJECT_CONFIGURATION));
         transformProgressListeners.addAll(getOrDefault(listeners, OperationType.TRANSFORM));
         testOutputProgressListeners.addAll(getOrDefault(listeners, OperationType.TEST_OUTPUT));
+        testMetadataProgressListeners.addAll(getOrDefault(listeners, OperationType.TEST_METADATA));
         fileDownloadListeners.addAll(getOrDefault(listeners, OperationType.FILE_DOWNLOAD));
         buildPhaseListeners.addAll(getOrDefault(listeners, OperationType.BUILD_PHASE));
         problemListeners.addAll(getOrDefault(listeners, OperationType.PROBLEMS));
+        rootBuildListeners.addAll(getOrDefault(listeners, OperationType.ROOT));
     }
 
     private static List<ProgressListener> getOrDefault(Map<OperationType, List<ProgressListener>> listeners, OperationType operationType) {
@@ -295,6 +327,9 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         if (!testOutputProgressListeners.isEmpty()) {
             operations.add(InternalBuildProgressListener.TEST_OUTPUT);
         }
+        if (!testMetadataProgressListeners.isEmpty()) {
+            operations.add(InternalBuildProgressListener.TEST_METADATA);
+        }
         if (!fileDownloadListeners.isEmpty()) {
             operations.add(InternalBuildProgressListener.FILE_DOWNLOAD);
         }
@@ -304,6 +339,9 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         if (!problemListeners.isEmpty()) {
             operations.add(InternalBuildProgressListener.PROBLEMS);
         }
+        if (!rootBuildListeners.isEmpty()) {
+            operations.add(InternalBuildProgressListener.ROOT);
+        }
         return operations;
     }
 
@@ -334,10 +372,14 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
             transformProgressListeners.getSource().statusChanged(event);
         } else if (event instanceof TestOutputEvent) {
             testOutputProgressListeners.getSource().statusChanged(event);
+        } else if (event instanceof TestMetadataEvent) {
+            testMetadataProgressListeners.getSource().statusChanged(event);
         } else if (event instanceof BuildPhaseProgressEvent) {
             buildPhaseListeners.getSource().statusChanged(event);
         } else if (event instanceof ProblemEvent) {
             problemListeners.getSource().statusChanged(event);
+        } else if (event instanceof FileDownloadProgressEvent || event instanceof DefaultStatusEvent) {
+            fileDownloadListeners.getSource().statusChanged(event);
         } else {
             // Everything else treat as a generic operation
             buildOperationProgressListeners.getSource().statusChanged(event);
@@ -363,19 +405,28 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
             broadcastTransformProgressEvent(progressEvent, (InternalTransformDescriptor) descriptor);
         } else if (descriptor instanceof InternalTestOutputDescriptor) {
             broadcastTestOutputEvent(progressEvent, (InternalTestOutputDescriptor) descriptor);
-        } else if (progressEvent instanceof InternalStatusEvent) {
-            broadcastStatusEvent((InternalStatusEvent) progressEvent);
+        } else if (descriptor instanceof InternalTestMetadataDescriptor) {
+            broadcastTestMetadataEvent(progressEvent, (InternalTestMetadataDescriptor) descriptor);
         } else if (descriptor instanceof InternalFileDownloadDescriptor) {
-            broadcastFileDownloadEvent(progressEvent, (InternalFileDownloadDescriptor) descriptor);
+            if (progressEvent instanceof InternalStatusEvent) {
+                broadcastStatusEvent((InternalStatusEvent) progressEvent);
+            } else {
+                broadcastFileDownloadEvent(progressEvent, (InternalFileDownloadDescriptor) descriptor);
+            }
         } else if (descriptor instanceof InternalBuildPhaseDescriptor) {
             broadcastBuildPhaseEvent(progressEvent, (InternalBuildPhaseDescriptor) descriptor);
         } else if (descriptor instanceof InternalProblemDescriptor) {
             broadcastProblemEvent(progressEvent, (InternalProblemDescriptor) descriptor);
+        } else if (descriptor instanceof InternalRootOperationDescriptor) {
+            broadcastRootBuildEvent(progressEvent);
         } else {
             broadcastGenericProgressEvent(progressEvent);
         }
     }
 
+    /*
+     * This represents a file download update event
+     */
     private void broadcastStatusEvent(InternalStatusEvent progressEvent) {
         OperationDescriptor descriptor = descriptorCache.get(progressEvent.getDescriptor().getId());
         if (descriptor == null) {
@@ -424,6 +475,13 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         }
     }
 
+    private void broadcastTestMetadataEvent(InternalProgressEvent event, InternalTestMetadataDescriptor descriptor) {
+        TestMetadataEvent metadataEvent = toTestMetadataEvent(event, descriptor);
+        if (metadataEvent != null) {
+            testMetadataProgressListeners.getSource().statusChanged(metadataEvent);
+        }
+    }
+
     private void broadcastProblemEvent(InternalProgressEvent progressEvent, InternalProblemDescriptor descriptor) {
         ProblemEvent problemEvent = toProblemEvent(progressEvent, descriptor);
         if (problemEvent != null) {
@@ -431,6 +489,16 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         }
     }
 
+    private void broadcastRootBuildEvent(InternalProgressEvent event) {
+        ProgressEvent progressEvent = toGenericProgressEvent(event);
+        if (progressEvent != null) {
+            rootBuildListeners.getSource().statusChanged(progressEvent);
+        }
+    }
+
+    /*
+     * Does not handle file download update events, see #broadcastStatusEvent for those
+     */
     private void broadcastFileDownloadEvent(InternalProgressEvent event, InternalFileDownloadDescriptor descriptor) {
         ProgressEvent progressEvent = toFileDownloadProgressEvent(event, descriptor);
         if (progressEvent != null) {
@@ -556,15 +624,31 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         return new DefaultTestOutputEvent(event.getEventTime(), clientDescriptor);
     }
 
+    private @Nullable TestMetadataEvent toTestMetadataEvent(InternalProgressEvent event, InternalTestMetadataDescriptor descriptor) {
+        if (event instanceof InternalTestMetadataEvent) {
+            return transformTestMetadata((InternalTestMetadataEvent) event, descriptor);
+        } else {
+            return null;
+        }
+    }
+
+    private TestMetadataEvent transformTestMetadata(InternalTestMetadataEvent event, InternalTestMetadataDescriptor descriptor) {
+        OperationDescriptor clientDescriptor = addDescriptor(event.getDescriptor(), toDescriptor(descriptor));
+        return new DefaultTestMetadataEvent(event.getEventTime(), clientDescriptor, event.getValues());
+    }
+
     private @Nullable ProblemEvent toProblemEvent(InternalProgressEvent progressEvent, InternalProblemDescriptor descriptor) {
         if (progressEvent instanceof InternalProblemEvent) {
             InternalProblemEvent problemEvent = (InternalProblemEvent) progressEvent;
             return createProblemEvent(problemEvent, descriptor);
+        } else if (progressEvent instanceof InternalProblemEventVersion2) {
+            InternalProblemEventVersion2 problemEvent = (InternalProblemEventVersion2) progressEvent;
+            return createProblemEvent(problemEvent, descriptor);
         }
         return null;
     }
 
-    private ProblemEvent createProblemEvent(InternalProblemEvent problemEvent, InternalProblemDescriptor descriptor) {
+    private @Nullable ProblemEvent createProblemEvent(InternalProblemEvent problemEvent, InternalProblemDescriptor descriptor) {
         InternalProblemDetails details = problemEvent.getDetails();
         OperationDescriptor parentDescriptor = getParentDescriptor(descriptor.getParentId());
 
@@ -573,15 +657,7 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
             return new DefaultSingleProblemEvent(
                 problemEvent.getEventTime(),
                 parentDescriptor,
-                toProblemCategory(basicProblemDetails.getCategory()),
-                toProblemSeverity(basicProblemDetails.getSeverity()),
-                toProblemLabel(basicProblemDetails.getLabel()),
-                toProblemDetails(basicProblemDetails.getDetails()),
-                toLocations(basicProblemDetails.getLocations()),
-                toDocumentationLink(basicProblemDetails.getDocumentationLink()),
-                toSolutions(basicProblemDetails.getSolutions()),
-                toAdditionalData(basicProblemDetails.getAdditionalData()),
-                toFailureContainer(basicProblemDetails)
+                toProblem(basicProblemDetails)
             );
         } else if (details instanceof InternalProblemAggregationDetailsV2) {
             InternalProblemAggregationDetailsV2 problemAggregationDetails = (InternalProblemAggregationDetailsV2) details;
@@ -589,27 +665,104 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
                 problemEvent.getEventTime(),
                 parentDescriptor,
                 new DefaultProblemAggregation(
-                    toProblemCategory(problemAggregationDetails.getCategory()),
-                    toProblemLabel(problemAggregationDetails.getLabel()),
-                    toProblemSeverity(problemAggregationDetails.getSeverity()),
-                    toDocumentationLink(problemAggregationDetails.getDocumentationLink()),
+                    toProblemDefinition(problemAggregationDetails.getLabel(), problemAggregationDetails.getCategory(), problemAggregationDetails.getSeverity(), problemAggregationDetails.getDocumentationLink()),
                     toProblemContextDetails(problemAggregationDetails.getProblems())));
 
         }
-
         return null;
     }
 
-    @Nonnull
-    private static DefaultProblemsOperationContext toSingleProblemContextDetail(InternalProblemContextDetails details) {
-        return new DefaultProblemsOperationContext(
-            toProblemDetails(details.getDetails()),
-            toLocations(details.getLocations()),
-            toSolutions(details.getSolutions()),
-            toAdditionalData(details.getAdditionalData()),
-            toFailureContainer(details.getFailure())
-        );
+    @NonNull
+    static List<ProblemSummary> toProblemIdSummaries(List<InternalProblemSummary> problemIdCounts) {
+        Map<ProblemId, List<InternalProblemSummary>> groupedSummaries = getGroupedMap(problemIdCounts);
+
+        List<ProblemSummary> problemSummaries = new ArrayList<>();
+        for (Map.Entry<ProblemId, List<InternalProblemSummary>> groupEntry : groupedSummaries.entrySet()) {
+            problemSummaries.add(new DefaultProblemSummary(groupEntry.getKey(), getCount(groupEntry)));
+        }
+        return problemSummaries;
     }
+
+    @NonNull
+    static Map<ProblemId, List<InternalProblemSummary>> getGroupedMap(List<InternalProblemSummary> problemIdCounts) {
+        Map<ProblemId, List<InternalProblemSummary>> groupedSummaries = new HashMap<>();
+        for (InternalProblemSummary internalSummary : problemIdCounts) {
+            ProblemId problemId = toProblemId(internalSummary.getProblemId());
+            getOrDefault(groupedSummaries, problemId).add(internalSummary);
+        }
+        return groupedSummaries;
+    }
+
+    @NonNull
+    private static List<InternalProblemSummary> getOrDefault(Map<ProblemId, List<InternalProblemSummary>> groupedSummaries, ProblemId problemId) {
+        List<InternalProblemSummary> internalProblemSummaries = groupedSummaries.get(problemId);
+        if (internalProblemSummaries == null) {
+            internalProblemSummaries = new ArrayList<>();
+            groupedSummaries.put(problemId, internalProblemSummaries);
+        }
+        return internalProblemSummaries;
+    }
+
+    static int getCount(Map.Entry<ProblemId, List<InternalProblemSummary>> groupEntry) {
+        int count = 0;
+        for (InternalProblemSummary internalProblemSummary : groupEntry.getValue()) {
+            count += internalProblemSummary.getCount();
+        }
+        return count;
+    }
+
+    private @Nullable ProblemEvent createProblemEvent(InternalProblemEventVersion2 problemEvent, InternalProblemDescriptor descriptor) {
+        InternalProblemDetailsVersion2 details = problemEvent.getDetails();
+        OperationDescriptor parentDescriptor = getParentDescriptor(descriptor.getParentId());
+
+        if (details instanceof InternalBasicProblemDetailsVersion3) {
+            InternalBasicProblemDetailsVersion3 basicProblemDetails = (InternalBasicProblemDetailsVersion3) details;
+            return new DefaultSingleProblemEvent(
+                problemEvent.getEventTime(),
+                parentDescriptor,
+                toProblem(basicProblemDetails)
+            );
+        } else if (details instanceof InternalProblemAggregationDetailsVersion3) {
+            InternalProblemAggregationDetailsVersion3 problemAggregationDetails = (InternalProblemAggregationDetailsVersion3) details;
+            return new DefaultProblemAggregationEvent(
+                problemEvent.getEventTime(),
+                parentDescriptor,
+                new DefaultProblemAggregation(
+                    toProblemDefinition(problemAggregationDetails.getDefinition()),
+                    toProblemContextDetails(problemAggregationDetails.getProblems())));
+
+        } else if (details instanceof InternalProblemSummariesDetails) {
+            InternalProblemSummariesDetails problemSummariesDetails = (InternalProblemSummariesDetails) details;
+            return new DefaultProblemSummariesEvent(problemEvent.getEventTime(), parentDescriptor,
+                toProblemIdSummaries(problemSummariesDetails.getProblemIdCounts()));
+        }
+        return null;
+    }
+
+    @NonNull
+    private static DefaultProblemsOperationContext toSingleProblemContextDetail(InternalProblemContextDetails details) {
+        if (details instanceof InternalProblemContextDetailsV2) {
+            InternalProblemContextDetailsV2 detailsV2 = (InternalProblemContextDetailsV2) details;
+            return new DefaultProblemsOperationContext(
+                toProblemDetails(detailsV2.getDetails()),
+                toLocations(detailsV2.getOriginLocations()),
+                toLocations(detailsV2.getContextualLocations()),
+                toSolutions(detailsV2.getSolutions()),
+                toAdditionalData(detailsV2.getAdditionalData()),
+                toFailure(detailsV2.getFailure())
+            );
+        } else {
+            return new DefaultProblemsOperationContext(
+                toProblemDetails(details.getDetails()),
+                toLocations(details.getLocations()),
+                ImmutableList.<Location>of(),
+                toSolutions(details.getSolutions()),
+                toAdditionalData(details.getAdditionalData()),
+                toFailure(details.getFailure())
+            );
+        }
+    }
+
     private static List<ProblemContext> toProblemContextDetails(List<InternalProblemContextDetails> problems) {
         ImmutableList.Builder<ProblemContext> result = builderWithExpectedSize(problems.size());
         for (InternalProblemContextDetails problem : problems) {
@@ -717,7 +870,7 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         return assertDescriptorType(type, cachedTestDescriptor);
     }
 
-    private <T extends OperationDescriptor> T assertDescriptorType(Class<T> type, OperationDescriptor descriptor) {
+    private static <T extends OperationDescriptor> T assertDescriptorType(Class<T> type, OperationDescriptor descriptor) {
         Class<? extends OperationDescriptor> descriptorClass = descriptor.getClass();
         if (!type.isAssignableFrom(descriptorClass)) {
             throw new IllegalStateException(String.format("Unexpected operation type. Required %s but found %s", type.getName(), descriptorClass.getName()));
@@ -784,31 +937,101 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         return new DefaultTestOutputOperationDescriptor(descriptor, parent, destination, message);
     }
 
-    private static @Nullable FailureContainer toFailureContainer(@Nullable InternalBasicProblemDetails problemDetails) {
-        if (!(problemDetails instanceof InternalBasicProblemDetailsVersion2)) {
-            return null;
-        }
-        return toFailureContainer(((InternalBasicProblemDetailsVersion2) problemDetails).getFailure());
+    private static Problem toProblem(InternalBasicProblemDetails basicProblemDetails) {
+        return new DefaultProblem(
+            toProblemDefinition(basicProblemDetails.getLabel(), basicProblemDetails.getCategory(), basicProblemDetails.getSeverity(), basicProblemDetails.getDocumentationLink()),
+            toContextualLabel(basicProblemDetails.getLabel().getLabel()),
+            toProblemDetails(basicProblemDetails.getDetails()),
+            toLocations(basicProblemDetails.getLocations()),
+            Collections.<Location>emptyList(),
+            toSolutions(basicProblemDetails.getSolutions()),
+            toAdditionalData(basicProblemDetails.getAdditionalData()),
+            toFailure(basicProblemDetails)
+        );
     }
 
-    @Nullable
-    private static FailureContainer toFailureContainer(@Nullable InternalFailure failure) {
-        if (failure == null) {
-            return null;
+    private static Problem toProblem(InternalBasicProblemDetailsVersion3 basicProblemDetails) {
+        List<InternalLocation> originLocations;
+        List<InternalLocation> contextualLocations;
+        if (basicProblemDetails instanceof InternalBasicProblemDetailsVersion4) {
+            originLocations = ((InternalBasicProblemDetailsVersion4) basicProblemDetails).getOriginLocations();
+            contextualLocations = ((InternalBasicProblemDetailsVersion4) basicProblemDetails).getContextualLocations();
+
+        } else {
+            originLocations = basicProblemDetails.getLocations();
+            contextualLocations = Collections.emptyList();
         }
-        return new DefaultFailureContainer(toFailure(failure));
+        return new DefaultProblem(
+            toProblemDefinition(basicProblemDetails.getDefinition()),
+            toContextualLabel(basicProblemDetails.getContextualLabel()),
+            toProblemDetails(basicProblemDetails.getDetails()),
+            toLocations(originLocations),
+            toLocations(contextualLocations),
+            toSolutions(basicProblemDetails.getSolutions()),
+            toAdditionalData(basicProblemDetails.getAdditionalData()),
+            toFailure(basicProblemDetails.getFailure())
+        );
+    }
+
+    private static ProblemDefinition toProblemDefinition(InternalProblemDefinition problemDefinition) {
+        return new DefaultProblemDefinition(
+            toProblemId(problemDefinition.getId()),
+            toProblemSeverity(problemDefinition.getSeverity()),
+            toDocumentationLink(problemDefinition.getDocumentationLink())
+        );
+    }
+
+    private static ProblemDefinition toProblemDefinition(InternalLabel label, InternalProblemCategory category, InternalSeverity severity, @Nullable InternalDocumentationLink documentationLink) {
+        return new DefaultProblemDefinition(
+            toProblemId(label, category),
+            toProblemSeverity(severity),
+            toDocumentationLink(documentationLink)
+        );
+    }
+
+    private static ProblemId toProblemId(InternalProblemId problemId) {
+        return new DefaultProblemId(problemId.getName(), problemId.getDisplayName(), toProblemGroup(problemId.getGroup()));
+    }
+
+    private static ProblemId toProblemId(InternalLabel label, InternalProblemCategory category) {
+        List<String> categories = new ArrayList<>();
+        categories.add(category.getCategory());
+        categories.addAll(category.getSubcategories());
+
+        return new DefaultProblemId(categories.remove(categories.size() - 1), label.getLabel(), toProblemGroup(categories));
+    }
+
+    private static @Nullable ProblemGroup toProblemGroup(List<String> groupNames) {
+        if (groupNames.isEmpty()) {
+            return null;
+        } else {
+            String groupName = groupNames.remove(groupNames.size() - 1);
+            return new DefaultProblemGroup(groupName, groupName, toProblemGroup(groupNames));
+        }
+    }
+
+    private static ProblemGroup toProblemGroup(InternalProblemGroup problemGroup) {
+        return new DefaultProblemGroup(problemGroup.getName(), problemGroup.getDisplayName(), problemGroup.getParent() == null ? null : toProblemGroup(problemGroup.getParent()));
     }
 
     private static AdditionalData toAdditionalData(InternalAdditionalData additionalData) {
+        if (additionalData instanceof InternalProxiedAdditionalData) {
+            Object proxy = ((InternalProxiedAdditionalData) additionalData).getProxy();
+            return new DefaultCustomAdditionalData(additionalData.getAsMap(), proxy);
+        }
+        if (additionalData == null) {
+            return new DefaultAdditionalData(Collections.<String, Object>emptyMap());
+        }
         return new DefaultAdditionalData(additionalData.getAsMap());
     }
 
-    private static ProblemCategory toProblemCategory(InternalProblemCategory category) {
-        return new DefaultCategory(category.getNamespace(), category.getCategory(), category.getSubcategories());
+    @Nullable
+    private static ContextualLabel toContextualLabel(@Nullable InternalContextualLabel contextualLabel) {
+        return contextualLabel == null ? null : new DefaultContextualLabel(contextualLabel.getContextualLabel());
     }
 
-    private static Label toProblemLabel(InternalLabel label) {
-        return new DefaultLabel(label.getLabel());
+    private static ContextualLabel toContextualLabel(@Nullable String contextualLabel) {
+        return contextualLabel == null ? null : new DefaultContextualLabel(contextualLabel);
     }
 
     private static Severity toProblemSeverity(InternalSeverity severity) {
@@ -839,7 +1062,7 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
     }
 
     private static DocumentationLink toDocumentationLink(@Nullable InternalDocumentationLink link) {
-        return link == null ? new DefaultDocumentationLink(null) : new DefaultDocumentationLink(link.getUrl());
+        return link == null || link.getUrl() == null ? null : new DefaultDocumentationLink(link.getUrl());
     }
 
     private static List<Solution> toSolutions(List<InternalSolution> solutions) {
@@ -854,7 +1077,7 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         if (details != null) {
             return new DefaultDetails(details.getDetails());
         }
-        return new DefaultDetails(null);
+        return null;
     }
 
     private Set<OperationDescriptor> collectDescriptors(Set<? extends InternalOperationDescriptor> dependencies) {
@@ -885,7 +1108,7 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         }
     }
 
-    private @Nullable FileDownloadResult toFileDownloadResult(InternalOperationResult result) {
+    private static @Nullable FileDownloadResult toFileDownloadResult(InternalOperationResult result) {
         InternalFileDownloadResult fileDownloadResult = (InternalFileDownloadResult) result;
         if (result instanceof InternalNotFoundFileDownloadResult) {
             return new NotFoundFileDownloadSuccessResult(result.getStartTime(), result.getEndTime());
@@ -899,7 +1122,7 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         return null;
     }
 
-    private @Nullable TestOperationResult toTestResult(InternalTestResult result) {
+    private static @Nullable TestOperationResult toTestResult(InternalTestResult result) {
         if (result instanceof InternalTestSuccessResult) {
             return new DefaultTestSuccessResult(result.getStartTime(), result.getEndTime());
         } else if (result instanceof InternalTestSkippedResult) {
@@ -1006,18 +1229,46 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
         }
     }
 
-    private static List<Failure> toFailures(List<? extends InternalFailure> causes) {
+    private static List<Failure> toFailures(@Nullable List<? extends InternalFailure> causes) {
         if (causes == null) {
             return null;
         }
         List<Failure> failures = new ArrayList<>();
         for (InternalFailure cause : causes) {
-            failures.add(toFailure(cause));
+            Failure f = toFailure(cause);
+            if (f != null) {
+                failures.add(f);
+            }
         }
         return failures;
     }
 
+    @Nullable
+    private static Failure toFailure(InternalBasicProblemDetails problemDetails) {
+        if (!(problemDetails instanceof InternalBasicProblemDetailsVersion2)) {
+            return null;
+        }
+        return toFailure(((InternalBasicProblemDetailsVersion2) problemDetails).getFailure());
+    }
+
+    @Nullable
     private static Failure toFailure(InternalFailure origFailure) {
+        if (origFailure == null) {
+            return null;
+        }
+        List<InternalBasicProblemDetailsVersion3> problemDetails = new ArrayList<>();
+        try {
+            problemDetails.addAll(origFailure.getProblems());
+        } catch (AbstractMethodError ignore) {
+            // Older Gradle versions don't have this method
+        }
+        List<Problem> clientProblems = new ArrayList<>(problemDetails.size());
+        for (InternalBasicProblemDetailsVersion3 problemDetail : problemDetails) {
+            if (problemDetail == null) { // Should not happen, but with some older snapshot versions we see this.
+                continue;
+            }
+            clientProblems.add(toProblem(problemDetail));
+        }
         if (origFailure instanceof InternalTestAssertionFailure) {
             if (origFailure instanceof InternalFileComparisonTestAssertionFailure) {
                 InternalTestAssertionFailure assertionFailure = (InternalTestAssertionFailure) origFailure;
@@ -1052,10 +1303,11 @@ public class BuildProgressListenerAdapter implements InternalBuildProgressListen
                 ((InternalTestFrameworkFailure) origFailure).getStacktrace()
             );
         }
-        return origFailure == null ? null : new DefaultFailure(
+        return new DefaultFailure(
             origFailure.getMessage(),
             origFailure.getDescription(),
-            toFailures(origFailure.getCauses()));
+            toFailures(origFailure.getCauses()),
+            clientProblems);
     }
 
     private static @Nullable List<AnnotationProcessorResult> toAnnotationProcessorResults(@Nullable List<InternalAnnotationProcessorResult> protocolResults) {
