@@ -17,7 +17,9 @@ package org.gradle.testfixtures.internal;
 
 import org.gradle.cache.FileLockManager;
 import org.gradle.cache.internal.CacheFactory;
+import org.gradle.internal.classpath.ClassPath;
 import org.gradle.internal.concurrent.ExecutorFactory;
+import org.gradle.internal.installation.CurrentGradleInstallation;
 import org.gradle.internal.instrumentation.agent.AgentStatus;
 import org.gradle.internal.logging.LoggingManagerFactory;
 import org.gradle.internal.logging.LoggingManagerInternal;
@@ -32,7 +34,7 @@ import org.gradle.internal.time.Clock;
 
 public class TestGlobalScopeServices extends GlobalScopeServices {
     public TestGlobalScopeServices() {
-        super(false, AgentStatus.disabled());
+        super(false, AgentStatus.disabled(), ClassPath.EMPTY, CurrentGradleInstallation.locate());
     }
 
     @Provides
